@@ -3,7 +3,6 @@ from nglui.easyviewer.utils import default_neuroglancer_base
 import pandas as pd
 import numpy as np
 from collections.abc import Collection
-from warnings import warn
 from IPython.display import HTML
 from .utils import bucket_of_values
 
@@ -14,7 +13,6 @@ DEFAULT_ANNO_LAYER = 'anno'
 DEFAULT_VIEW_KWS = {'layout': 'xy-3d',
                     'zoom_image': 2,
                     'show_slices': False,
-                    'zoom_image': 8,
                     'zoom_3d': 2000,
                     }
 
@@ -115,6 +113,10 @@ class ImageLayerConfig(LayerConfigBase):
         If True, makes the layer active in neuroglancer. Default is False.
     contrast_controls : bool, optional
         If True, gives the layer a user-controllable brightness and contrast shader. Default is False.
+    black : float, optional
+        If contrast_controls is True, sets the default black level. Default is 0.0.
+    white : float, optional
+        If contrast_controls is True, sets the default white level. Default is 1.0.
     """
 
     def __init__(self,
@@ -122,15 +124,21 @@ class ImageLayerConfig(LayerConfigBase):
                  name=DEFAULT_IMAGE_LAYER,
                  active=False,
                  contrast_controls=False,
+                 black=0.0,
+                 white=1.0,
                  ):
         super(ImageLayerConfig, self).__init__(
             name=name, type='image', source=source, color=None, active=active)
-        self._contrast_controls = contrast_controls 
+        self._contrast_controls = contrast_controls
+        self._black = black
+        self._white = white
 
     def _specific_rendering(self, viewer, data):
         viewer.add_image_layer(self.name, self.source)
         if self._contrast_controls:
-            viewer.add_contrast_shader(self.name)
+            viewer.add_contrast_shader(
+                self.name, black=self._black, white=self._white)
+
 
 class SelectionMapper(object):
     """Class for configuring object selections based on root id
@@ -283,6 +291,9 @@ class AnnotationLayerConfig(LayerConfigBase):
         Name of a linked segmentation layer for selected ids. By default, None
     mapping_rules : PointMapper, LineMapper, SphereMapper or list, optional
         One rule or a list of rules mapping data to annotations. By default, []
+    array_data : bool, optional
+        If True, allows simple mapping where one or more arrays are passed instead of a dataframe.
+        Only allows basic annotation creation, no tags, linked segmentations, or other rich features.
     tags : list, optional
         List of tags for the layer.
     active : bool, optional
@@ -294,13 +305,22 @@ class AnnotationLayerConfig(LayerConfigBase):
                  color=None,
                  linked_segmentation_layer=None,
                  mapping_rules=[],
+                 array_data=False,
                  tags=None,
-                 active=True):
+                 active=True,
+                 ):
         super(AnnotationLayerConfig, self).__init__(
             name=name, type='annotation', color=color, source=None, active=active)
         self._config['linked_segmentation_layer'] = linked_segmentation_layer
         if issubclass(type(mapping_rules), AnnotationMapperBase):
             mapping_rules = [mapping_rules]
+        if array_data is True:
+            if len(mapping_rules) > 1:
+                raise ValueError(
+                    "Only one mapping rule can be set using array data")
+            for mr in mapping_rules:
+                mr.array_data = array_data
+        self._array_data = array_data
         self._annotation_map_rules = mapping_rules
         self._tags = tags
 
@@ -309,6 +329,10 @@ class AnnotationLayerConfig(LayerConfigBase):
         return self._config.get('linked_segmentation_layer', None)
 
     def _specific_rendering(self, viewer, data):
+        if self._array_data:
+            if len(self._annotation_map_rules) > 0:
+                data = pd.DataFrame(
+                    data={self._annotation_map_rules[0].data_columns[0]: data.tolist()})
         viewer.add_annotation_layer(self.name,
                                     color=self.color,
                                     linked_segmentation_layer=self.linked_segmentation_layer,
@@ -330,14 +354,17 @@ class AnnotationMapperBase(object):
                  description_column,
                  linked_segmentation_column,
                  tag_column,
+                 group_column,
                  set_position,
                  ):
 
         self._config = dict(type=type,
                             data_columns=data_columns,
+                            array_data=False,
                             description_column=description_column,
                             linked_segmentation_column=linked_segmentation_column,
                             tag_column=tag_column,
+                            group_column=group_column,
                             set_position=set_position,
                             )
         self._tag_map = None
@@ -362,6 +389,10 @@ class AnnotationMapperBase(object):
     def tag_column(self):
         return self._config.get('tag_column', None)
 
+    @property
+    def group_column(self):
+        return self._config.get('group_column', None)
+
     @property
     def set_position(self):
         return self._config.get('set_position', False)
@@ -380,6 +411,22 @@ class AnnotationMapperBase(object):
         else:
             self._tag_map = {tag: ii+1 for ii, tag in enumerate(tag_list)}
 
+    @property
+    def array_data(self):
+        return self._config.get('array_data', False)
+
+    @array_data.setter
+    def array_data(self, new_array_data):
+        self._config['array_data'] = new_array_data
+        if new_array_data:
+            self._config['data_columns'] = self._default_array_data_columns()
+            self._config['description_column'] = None
+            self._config['linked_segmentation_column'] = None
+            self._config['tag_column'] = None
+
+    def _default_array_data_columns(self):
+        return []
+
     def _assign_tags(self, data):
         if self.tag_column is not None:
             anno_tags = []
@@ -414,6 +461,17 @@ class AnnotationMapperBase(object):
             descriptions = [None for x in range(len(data))]
         return descriptions
 
+    def _add_groups(self, groups, annos):
+        vals, inverse = np.unique(groups, return_inverse=True)
+        inv_inds = np.flatnonzero(~np.isnan(vals))
+        group_annos = []
+        for ii in inv_inds:
+            anno_to_group = [annos[jj] for jj in np.flatnonzero(inverse == ii)]
+            group_annos.append(annotation.group_annotations(
+                anno_to_group, return_all=False))
+        annos.extend(group_annos)
+        return annos
+
     def _get_position(self, data):
         if len(data) > 0 and self.set_position is True:
             return list(data[self.data_columns[0]].iloc[0])
@@ -426,35 +484,49 @@ class PointMapper(AnnotationMapperBase):
 
     Parameters
     ----------
-    point_column : str
+    point_column : str, optional
         Column name with 3d position data
+    array_data : bool, optional
+        If True, the expected data is a Nx3 numpy array. This will only work for point positions,
+        to use any other features you must build a dataframe.
     decription_column : str, optional
         Column name with string data for annotation descriptions
     linked_segmentation_column : str, optional
         Column name for root ids to link to annotations
     tag_column : str, optional
-        Column name for categorical tag data. Tags must match those set in the annotation layer.
+        Column name for categorical tag data. Tags must match those set in the
+        annotation layer.
+    group_column : str, optional
+        Column name for grouping data. Data in this row should be numeric with possible NaNs.
+        Rows with the same non-NaN value will be collected into a grouped annotation.
     set_position : bool, optional
-        If set to True, moves the position to center on the first point in the data.
+        If set to True, moves the position to center on the first point in the
+        data.
     """
 
     def __init__(self,
-                 point_column,
+                 point_column=None,
                  description_column=None,
                  linked_segmentation_column=None,
                  tag_column=None,
+                 group_column=None,
                  set_position=False,
                  ):
-
         super(PointMapper, self).__init__(type='point',
                                           data_columns=[point_column],
                                           description_column=description_column,
                                           linked_segmentation_column=linked_segmentation_column,
                                           tag_column=tag_column,
+                                          group_column=group_column,
                                           set_position=set_position,
                                           )
 
+    def _default_array_data_columns(self):
+        return ['pt']
+
     def _render_data(self, data):
+        if self.array_data:
+            data = pd.DataFrame(data={'pt': np.array(data).tolist()})
         col = self.data_columns[0]
         relinds = ~pd.isnull(data[col])
 
@@ -468,6 +540,11 @@ class PointMapper(AnnotationMapperBase):
                                              linked_segmentation=ls,
                                              tag_ids=t) for
                  pt, d, ls, t in zip(pts, descriptions, linked_segs, tags)]
+
+        if self.group_column is not None:
+            groups = data[self.group_column].values[relinds]
+            annos = self._add_groups(groups, annos)
+
         return annos
 
 
@@ -477,38 +554,50 @@ class LineMapper(AnnotationMapperBase):
     Parameters
     ----------
     point_column_a : str
-        Column name with 3d position data for the first point of the line
+        Column name with 3d position data for the first point of the line.
+        Must be set if array_data is False (the default)
     point_column_b : str
-        Column name with 3d position data for the first point of the line
+        Column name with 3d position data for the first point of the line.
+        Must be set if array_data is False (the default)
     decription_column : str, optional
         Column name with string data for annotation descriptions
     linked_segmentation_column : str, optional
         Column name for root ids to link to annotations
     tag_column : str, optional
         Column name for categorical tag data. Tags must match those set in the annotation layer.
+    group_column : str, optional
+        Column name for grouping data. Data in this row should be numeric with possible NaNs.
+        Rows with the same non-NaN value will be collected into a grouped annotation.
     set_position : bool, optional
         If set to True, moves the position to center on the first point in the data (using point_column_a).
     """
 
     def __init__(self,
-                 point_column_a,
-                 point_column_b,
+                 point_column_a=None,
+                 point_column_b=None,
                  description_column=None,
                  linked_segmentation_column=None,
                  tag_column=None,
+                 group_column=None,
                  set_position=False,
                  ):
-
         super(LineMapper, self).__init__(type='line',
                                          data_columns=[
                                               point_column_a, point_column_b],
                                          description_column=description_column,
                                          linked_segmentation_column=linked_segmentation_column,
                                          tag_column=tag_column,
+                                         group_column=group_column,
                                          set_position=set_position,
                                          )
 
+    def _default_array_data_columns(self):
+        return ['pt_a', 'pt_b']
+
     def _render_data(self, data):
+        if self.array_data:
+            data = pd.DataFrame(
+                data={'pt_a': data[0].tolist(), 'pt_b': data[1].tolist()})
         colA, colB = self.data_columns
 
         relinds = np.logical_and(~pd.isnull(
@@ -524,6 +613,11 @@ class LineMapper(AnnotationMapperBase):
                                             linked_segmentation=ls,
                                             tag_ids=t) for
                  ptA, ptB, d, ls, t in zip(ptAs, ptBs, descriptions, linked_segs, tags)]
+
+        if self.group_column is not None:
+            groups = data[self.group_column].values[relinds]
+            annos = self._add_groups(groups, annos)
+
         return annos
 
 
@@ -542,31 +636,41 @@ class SphereMapper(AnnotationMapperBase):
         Column name for root ids to link to annotations
     tag_column : str, optional
         Column name for categorical tag data. Tags must match those set in the annotation layer.
+    group_column : str, optional
+        Column name for grouping data. Data in this row should be numeric with possible NaNs.
+        Rows with the same non-NaN value will be collected into a grouped annotation.
     set_position : bool, optional
         If set to True, moves the position to center on the first point in the data.
     """
 
     def __init__(self,
-                 center_column,
-                 radius_column,
+                 center_column=None,
+                 radius_column=None,
                  description_column=None,
                  linked_segmentation_column=None,
                  tag_column=None,
+                 group_column=None,
                  z_multiplier=0.1,
                  set_position=False,
                  ):
-
         super(SphereMapper, self).__init__(type='sphere',
                                            data_columns=[
                                                center_column, radius_column],
                                            description_column=description_column,
                                            linked_segmentation_column=linked_segmentation_column,
                                            tag_column=tag_column,
+                                           group_column=group_column,
                                            set_position=set_position,
                                            )
         self._z_multiplier = z_multiplier
 
+    def _default_array_data_columns(self):
+        return ['ctr_pt', 'rad']
+
     def _render_data(self, data):
+        if self.array_data:
+            data = pd.DataFrame(
+                data={'ctr_pt': data[0].tolist(), 'rad': data[1]})
         col_ctr, col_rad = self.data_columns
         relinds = np.logical_and(~pd.isnull(
             data[col_ctr]), ~pd.isnull(data[col_rad]))
@@ -582,6 +686,11 @@ class SphereMapper(AnnotationMapperBase):
                                               tag_ids=t,
                                               z_multiplier=self._z_multiplier) for
                  pt, r, d, ls, t in zip(pts, rs, descriptions, linked_segs, tags)]
+
+        if self.group_column is not None:
+            groups = data[self.group_column].values[relinds]
+            annos = self._add_groups(groups, annos)
+
         return annos
 
 
